import argparse
import importlib
from .jumps import *


def add_argument_jump(parser: argparse.ArgumentParser, *flags, dest: str = 'jump') -> None:
    if len(flags) <= 0:
        flags = ['-j', '--jump']
    parser.add_argument(
        *flags, dest=dest, action='append', required=False, default=[],
        help=f'A jump! (a tuple: <name>,<message2querys>,<results2message>).'
    )


def parse_args_jump(parser: argparse.ArgumentParser, jump_dest: str = 'jump'):
    args = parser.parse_args()
    jump_list = []
    for jump_s in args.__getattribute__(jump_dest):
        jump = eval(jump_s)
        if isinstance(jump, tuple):
            jump_list.append(jump)
    return jump_list



import argparse
import logging
from neo4j import GraphDatabase
from telegram.ext import CommandHandler, ApplicationBuilder
from .arg import add_argument_jump, parse_args_jump
from .jump import Rabbot
logging.basicConfig(
    format='%(asctime)s - %(name)s - %(levelname)s - %(message)s',
    level=logging.INFO
)

parser = argparse.ArgumentParser()

parser.add_argument("--auth", type=str, default=None, help=f'Auth to neo4j database.')
parser.add_argument("--uri", type=str, required=True, help=f'URI to neo4j database.')
parser.add_argument("--token", type=str, required=True, help=f'Telegram bot token.')
add_argument_jump(parser)


args = parser.parse_args()
jump_list = parse_args_jump(parser)


async def start(update, context):
    await context.bot.send_message(chat_id=update.effective_chat.id, text="I'm a rabbit jumping in citation database")

application = ApplicationBuilder().token(args.token).build()
application.add_handler(CommandHandler('start', start))
with GraphDatabase.driver(args.uri, auth=args.auth) as driver:
    with driver.session() as session:
        rabbot = Rabbot(app=application, session=session)
        for name, message2query, result2message in jump_list:
            rabbot.add_jump(name, message2query, result2message)
        application.run_polling()



import asyncio
import re
from typing import Callable, Tuple, Dict, List
from neo4j import Session
from telegram import Update, InlineKeyboardMarkup
from telegram.ext import filters, Application, ContextTypes, CommandHandler, MessageHandler
from telegram.constants import ParseMode


class Rabbot:
    def __init__(self, app: Application, session: Session) -> None:
        self.app = app
        self.session = session
        self.bot_name = None

    async def _fetch_name(self):
        me = await self.app.bot.get_me()
        self.bot_name = me.name

    def add_jump(self, name: str, message2querys: Callable[[str], List[Tuple[str, Dict]]], results2message: Callable[[List], Tuple[str, InlineKeyboardMarkup]]):
        async def handler(text: str, update: Update, context: ContextTypes.DEFAULT_TYPE):
            querys = message2querys(text)
            if not querys or len(querys) <= 0:
                return
            results = []
            for query, kwargs in querys:
                results.append(self.session.execute_read(lambda tx: tx.run(query, **kwargs).values()))
            message, reply_markup = results2message(results)
            await update.message.reply_text(text=message, reply_to_message_id=update.message.id, reply_markup=reply_markup, parse_mode=ParseMode.HTML)
        command_handler = CommandHandler(name, lambda update, context: handler(update.message.text, update, context))
        self.app.add_handler(command_handler)

        if self.bot_name is None:
            loop = asyncio.get_event_loop()
            loop.run_until_complete(self._fetch_name())

        async def wrap_handler(update: Update, context: ContextTypes.DEFAULT_TYPE):
            split = re.findall(f"^{self.bot_name} +(/{name} +.*)", update.message.text)
            if len(split) > 0:
                await handler(split[0], update, context)
        message_handler = MessageHandler(filters.Regex(f"^{self.bot_name} +/{name}"), wrap_handler)
        self.app.add_handler(message_handler)



#!/usr/bin/env python
# coding: utf-8

from setuptools import setup

with open("README.md", "r", encoding='utf8') as fh:
    long_description = fh.read()

package_dir = {
    'citation_rabbot': 'citation_rabbot'
}

setup(
    name='citation-rabbot',
    version='1.1',
    author='yindaheng98',
    author_email='yindaheng98@gmail.com',
    url='https://github.com/yindaheng98/citation-rabbot',
    description=u'A telegram bot jumping in your citation database like a rabbit!',
    long_description=long_description,
    long_description_content_type="text/markdown",
    package_dir=package_dir,
    packages=[key for key in package_dir],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'python-telegram-bot>=20.7',
        'neo4j>=5.15.0'
    ],
)
